#!/usr/bin/perl
# get_quotas.pl - No longer uses perl-Quota, but 'quota' instead.

my $login = (getpwuid $>);
die "must run as root" if $login ne 'root';

use lib qw(/usr/sausalito/perl);
use Base::HomeDir qw(homedir_get_group_dir);
use Getopt::Long;
use CCE;
use Unix::PasswdFile;

# Debugging switch:
$DEBUG = "0";
if ($DEBUG)
{
        use Sys::Syslog qw( :DEFAULT setlogsock);
        use Data::Dumper;
}

my $site = '';
my $sort = '';
my $descending = '';
my $help = '';
my $users_only = 1;
my $sites_only = '';
my $pw = new Unix::PasswdFile "/etc/passwd";

GetOptions( 'site=s' => \$site, 
            'sort=s' => \$sort, 
            'descending' => \$descending, 
            'ascending' => sub { $descending = 0 },
            'help' => \$help, 
            'users' => sub {$users_only = 1; $sites_only = 0}, 
            'sites' => sub { $sites_only = 1; $users_only = 0 }
            );

if ($help) {
    print "Usage: get_quotas.pl [ --users ] [ --sites ] \n\t\t\t[ --sort=type ] [ --site=name ] [ --descending ] [ --ascending] [ --help ]\n";
    print " --users\t Get quotas for users. Default.\n";
    print " --sites\t Get quotas for sites.\n";
    print " --sort=type\t Type is one of 'quota', 'usage', 'name'. \n\t\t\tIf none are specified, defaults to 'name'.\n";
    print " --site=name\t Return the quotas for users that are on site called 'name'.\n\t\t\tName is the 'name' property is CCE of the site you wish to find. \n\t\t\tUsually, it's 'siteN' where N is some number. \n\t\t\tIf this option is not present, \n\t\t\tthe script will return all users on all sites\n";
    print " --ascending\t Sort in ascending order. Default.\n";
    print " --descending\t Sort in descending order.\n";
    exit;
}

my (@items, %quotas) = ();

# Get all quotas via 'repquota' from the 'quota' RPM. Nees to be v4.0.4 
# or better as the '-O csv' has only recently been added.
$raw_userquota = `LC_ALL=C /usr/sbin/repquota -au -O csv|grep -v ^#|grep -v ^User`;
chomp $raw_userquota;
@uq = split /\n/, $raw_userquota;
foreach my $x (@uq) {
    chomp($x);
    next if $x =~ /^\s*$/;               # skip blank lines
    my (@row) = split (/,/, $x);
    $USERQUOTA{$row[0]} = { 'BlockUsed' => $row[3], 'BlockSoftLimit' => $row[4], 'BlockHardLimit' => $row[5], 'FileSoftLimit' => $row[8], 'FileHardLimit' => $row[9]};
}

$raw_groupquota = `LC_ALL=C /usr/sbin/repquota -ag -O csv|grep -v ^#|grep -v ^Group`;
chomp $raw_groupquota;
@gq = split /\n/, $raw_groupquota;
@found_groups = ();
foreach my $x (@gq) {
    chomp($x);
    next if $x =~ /^\s*$/;               # skip blank lines
    my (@row) = split (/,/, $x);
    push @found_groups, $row[0];
    $GROUPQUOTA{$row[0]} = { 'BlockUsed' => $row[3], 'BlockSoftLimit' => $row[4], 'BlockHardLimit' => $row[5], 'FileSoftLimit' => $row[8], 'FileHardLimit' => $row[9]};
}

if ($sites_only) {
    # return site quotas
    @items = sites();
    %quotas = siteusage();
} elsif ($users_only && $site) {
    # return user quotas on site $site
    @items = site_users($site);
    %quotas = userusage();
} elsif ($users_only && !$site) {
    # return user quotas on all sites
    @items = all_users();
    %quotas = userusage();
}

my (@results) = ();

foreach my $item (@items) {
    push @results, [ $item, $quotas{$item}{used},  $quotas{$item}{quota} ];
}

if ($sort eq "usage") {
    @results = sort { @{$a}[1] <=> @{$b}[1] } @results;
} elsif ($sort eq "quota") {
    @results = sort { if (@{$a}[2] == 0) { return 1; } # no quota = unlimited
    elsif (@{$b}[2] == 0) { return -1; } 
    else { return (@{$a}[2] <=> @{$b}[2]) } } @results;
} else {
    # sort == name
    @results = sort { @{$a}[0] cmp @{$b}[0] } @results;
}


if ($descending) {
    @results = reverse(@results);
}

my @output = ();
foreach my $user (@results) {
    print join ("\t", @{$user});
    print "\n";
}

# arg $site="site1" find all CCE users in the site $site
sub site_users {
    # if site is passed in, find all users on that site 
    my $site = shift;
    my $site_ok;

    my $cce = new CCE;
    $cce->connectuds();
    my ($oid) = $cce->find('Vsite', { 'name' => $site });
    if (!$oid) {
        &debug_msg("Couldn't find site $site in CCE\n");
        print STDERR "Couldn't find site $site in CCE\n";
        return;
    }
    ($site_ok, $site) = $cce->get($oid);

    my $sitedir = $site->{basedir};
    
    opendir(USERS, "$sitedir/home/users");
    my @users = grep !/^\./, readdir(USERS);
    closedir(USERS);

    return @users;
  
}

# find all CODB based users
sub all_users {

    # Rewritten from scratch by mstauber

    my ($name, $null, $all_gid, $user_gid, $dir);

    # all CCE users are NO LONGER in the "users" group - so we have do do it differently:
    my @all_users = ();

    foreach $name ($pw->users) {
        my $uid = $pw->uid($name);
        my $user_gid = $pw->gid($name);
        my $dir = $pw->home($name);
        my @groupworkaround = split(/\//, $dir);

        if ((($uid >= 500)&&($uid != 65534)) && ($groupworkaround[5] ne "logs")) {
            push @all_users, $name;
        }
    }
    return @all_users;
}

sub in_array {
    my ($arr,$search_for) = @_;
    my %items = map {$_ => 1} @$arr; # create a hash out of the array values
    return (exists($items{$search_for}))?1:0;
}

# Get a list of all Vsites. Fast and w/o using CCEd:
sub sites {
    my @sites = ();
    # This dirty one here gets us all directories from /home/.sites/:
    $sitedirs = `LC_ALL=C /usr/bin/tree -L 1 -d /home/.sites/|/usr/bin/grep "\s"|/usr/bin/awk '{print \$2}'|/usr/bin/grep -v ^server|grep -v directories`;
    chomp($sitedirs);
    @sd = split /\n/, $sitedirs;
    foreach my $x (@sd) {
        chomp($x);
        $fullDir = '/home/.sites/' . $x;
        #print "fullDir: $fullDir\n";
        if (-d $fullDir) {
            if (in_array(\@found_groups, $x)) {
                push @sites, $x;
            }            
        }
    }
    return @sites;
}

# Get Vsite quota usage:
sub siteusage {
    my %hash = ();
    # This dirty one here gets us all directories from /home/.sites/:
    $sitedirs = `LC_ALL=C /usr/bin/tree -L 1 -d /home/.sites/|/usr/bin/grep "\s"|/usr/bin/awk '{print \$2}'|/usr/bin/grep -v ^server|grep -v directories`;
    chomp($sitedirs);
    @sd = split /\n/, $sitedirs;
    foreach my $x (@sd) {
        chomp($x);
        $fullDir = '/home/.sites/' . $x;
        #print "fullDir: $fullDir\n";
        if (-d $fullDir) {
            if (in_array(\@found_groups, $x)) {
                $hash{$x}{used} = $GROUPQUOTA{$x}{BlockUsed};
                $hash{$x}{quota} = $GROUPQUOTA{$x}{BlockSoftLimit};
            }
        }
    }
    return %hash;
}

sub userusage {
    
    # Rewritten by mstauber

    my %hash = ();
    my ($name, $null, $uid, $user_gid, $all_gid, $dir);

    # all CCE users are NO LONGER in the "users" group - so we have to do it differently:

    foreach $name ($pw->users) {
        my $uid = $pw->uid($name);
        my $user_gid = $pw->gid($name);
        my $dir = $pw->home($name);
        my @groupworkaround = split(/\//, $dir);

        # Ignore all users with an UID below 500 and also the SITEXX-logs users:
        if (($uid < 500)||($uid == 65534)) {
            next;
        }
        elsif ($groupworkaround[5] eq "logs") {
            next;
        }
        elsif ($name eq "nfsnobody") {
            next;
        }
        else {
            $hash{$name}{used} = $USERQUOTA{$name}{BlockUsed};
            $hash{$name}{quota} = $USERQUOTA{$name}{BlockSoftLimit};
        }
    }
    return %hash;
}

sub debug_msg {
    if ($DEBUG) {
        my $msg = shift;
        setlogsock('unix');
        openlog($0,'','user');
        syslog('info', "$ARGV[0]: $msg");
        closelog;
    }
}

# 
# Copyright (c) 2008-2019 Michael Stauber, SOLARSPEED.NET
# Copyright (c) 2008-2019 Team BlueOnyx, BLUEONYX.IT
# Copyright (c) 2003 Sun Microsystems, Inc. 
# All Rights Reserved.
# 
# 1. Redistributions of source code must retain the above copyright 
#     notice, this list of conditions and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright 
#     notice, this list of conditions and the following disclaimer in 
#     the documentation and/or other materials provided with the 
#     distribution.
# 
# 3. Neither the name of the copyright holder nor the names of its 
#     contributors may be used to endorse or promote products derived 
#     from this software without specific prior written permission.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE 
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT 
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
# POSSIBILITY OF SUCH DAMAGE.
# 
# You acknowledge that this software is not designed or intended for 
# use in the design, construction, operation or maintenance of any 
# nuclear facility.
#